package net.wanji.opt.common;

import lombok.Data;

import java.time.Duration;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

public class TimeIntervalMerger {

    // 定义时间区间类
    @Data
    static class TimeInterval {
        private LocalTime start;
        private LocalTime end;

        public TimeInterval(LocalTime start, LocalTime end) {
            this.start = start;
            this.end = end;
        }

        // 判断两个区间是否重叠
        public boolean overlapsWith(TimeInterval other) {
            return !this.end.isBefore(other.start) && !other.end.isBefore(this.start);
        }

        // 合并两个区间
        public TimeInterval mergeWith(TimeInterval other) {
            return new TimeInterval(
                    this.start.isBefore(other.start) ? this.start : other.start,
                    this.end.isAfter(other.end) ? this.end : other.end
            );
        }

        // 计算时间段的持续时间（以秒为单位）
        public long getDurationInSeconds() {
            return Duration.between(start, end).getSeconds();
        }
    }

    // 定义支持单个数字小时部分的 DateTimeFormatter
    private static final DateTimeFormatter TIME_FORMATTER = DateTimeFormatter.ofPattern("H:mm");

    public static String timeString(List<String> timeStrings) {
        // 解析时间字符串为 TimeInterval 对象
        List<TimeInterval> intervals = timeStrings.stream()
                .map(timeStr -> {
                    String[] parts = timeStr.split("~");
                    LocalTime start = LocalTime.parse(parts[0].trim(), TIME_FORMATTER);
                    LocalTime end = LocalTime.parse(parts[1].trim(), TIME_FORMATTER);
                    return new TimeInterval(start, end);
                })
                .sorted(Comparator.comparing(TimeInterval::getStart)) // 按开始时间排序
                .collect(Collectors.toList());

        // 合并重叠的区间
        List<TimeInterval> mergedIntervals = new ArrayList<>();
        for (TimeInterval current : intervals) {
            if (mergedIntervals.isEmpty() || !current.overlapsWith(mergedIntervals.get(mergedIntervals.size() - 1))) {
                mergedIntervals.add(current); // 如果不重叠，直接添加
            } else {
                // 如果重叠，合并当前区间与最后一个区间
                TimeInterval last = mergedIntervals.get(mergedIntervals.size() - 1);
                mergedIntervals.set(mergedIntervals.size() - 1, last.mergeWith(current));
            }
        }

        // 按时间段长度从大到小排序
        mergedIntervals.sort((t1, t2) -> Long.compare(t2.getDurationInSeconds(), t1.getDurationInSeconds()));

        // 截取前三个时间段
        List<TimeInterval> topThreeIntervals = mergedIntervals.stream()
                .limit(3)
                .sorted(Comparator.comparing(TimeInterval::getStart)) // 按开始时间重新排序
                .collect(Collectors.toList());

        // 拼接结果字符串
        StringBuilder result = new StringBuilder();
        for (int i = 0; i < topThreeIntervals.size(); i++) {
            TimeInterval interval = topThreeIntervals.get(i);
            if (i > 0) {
                result.append(",");
            }
            result.append(interval.getStart()).append("~").append(interval.getEnd());
        }

        return result.toString();
    }

    // 测试代码
    public static void main(String[] args) {
        List<String> timeStrings = Arrays.asList(
                "7:00~8:00",
                "12:00~13:30",
                "17:30~18:40",
                "7:30~9:00",
                "17:00~18:00"
        );

        String result = timeString(timeStrings);
        System.out.println(result); // 输出结果
    }
}